@@ -1,6 +1,6 @@
 # Perlbug bug record handler
 # (C) 1999 Richard Foley RFI perlbug@rfi.net
-# $Id: Fixed.pm,v 1.8 2001/08/20 18:57:31 mstevens Exp $
+# $Id: Fixed.pm,v 1.9 2001/12/01 15:24:42 richardf Exp $
 #
 
 =head1 NAME
@@ -12,7 +12,7 @@ Perlbug::Object::Fixed - Fixed in Version handler
 package Perlbug::Object::Fixed;
 use strict;
 use vars qw($VERSION @ISA);
-$VERSION = do { my @r = (q$Revision: 1.8 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; 
+$VERSION = do { my @r = (q$Revision: 1.9 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; 
 $|=1;
 
 
@@ -20,14 +20,14 @@ $|=1;
 
 Perlbug B<Fixed> in B<Version> class handler
 
-For inherited methods, see L<Perlbug::Object::Version>
+For inherited methods, see L<Perlbug::Object>
 
 =cut
 
 use Data::Dumper;
 use Perlbug::Base;
-use Perlbug::Object::Version;
-@ISA = qw(Perlbug::Object::Version); 
+use Perlbug::Object;
+@ISA = qw(Perlbug::Object); 
 
 
 =head1 SYNOPSIS
@@ -56,9 +56,10 @@ sub new {
     my $class = ref($proto) || $proto; 
 	my $o_base = (ref($_[0])) ? shift : Perlbug::Base->new;
 
-	my $self = Perlbug::Object::Version->new( $o_base, 
-		'hint'		=> 'Fixed',
-		# 'from'		=> [qw(bug patch test)],
+	my $self = Perlbug::Object->new( $o_base, 
+		'name'			=> 'Fixed',
+		'from'			=> [qw(bug)],
+		'prejudicial'	=> 1,
 		@_,
 	);
 
